%Function M-file: vafdtd_animation.m
%
%Usage		: vafdtd_animation(sn, str, pmin, pmax, ratio, dframe[, xnx, xmx, ynx, ymx])
%Version	: 1.1.04 (2024.10.6)
%
%sn			: Surf Number
%str		: Sequence File Name
%pmin		: Minimum Sound Pressure
%pmax		: Maximum Sound Pressure
%ratio		: Ratio of Multiplying Displacement
%dframe		: Skip Frame Number
%xnx		: x-directional Minimum Position
%xmx		: x-directional Maximum Position
%ynx		: y-directional Minimum Position
%ymx		: y-directional Maximum Position

function vafdtd_animation(sn, str, pmin, pmax, ratio, dframe, xnx, xmx, ynx, ymx);

cscale	= 2048;

fid		= fopen([str '.ini'], 'r');
param	= fread(fid, [inf], 'float64');
fclose(fid);

dt			= param(1);
ix			= param(2);
jx			= param(3);
kx			= param(4);
tx			= param(5);
dfr			= param(6);
minX		= param(7);
maxX		= param(8);
minY		= param(9);
maxY		= param(10);
minZ		= param(11);
maxZ		= param(12);
pmlMinX		= param(13);
pmlMaxX		= param(14);
pmlMinY		= param(15);
pmlMaxY		= param(16);
pmlMinZ		= param(17);
pmlMaxZ		= param(18);
rptCount	= param(19);
for ii = 0: 1: rptCount - 1;
	rpts(ii+1)	= param(20+ii);
end;
srfCount	= param(20+rptCount*2);
for ii = 0: 1: srfCount - 1;
	srfs(ii+1)	= param(21+rptCount*2+ii);
end;
x	= zeros(ix, 1);
y	= zeros(jx, 1);
z	= zeros(kx, 1);
x(1)		= param(20+rptCount*2+srfCount+1) / 2 + pmlMinX;
for ii = 2: 1: ix
	x(ii)	= param(20+rptCount*2+srfCount+ii) / 2 + x(ii-1) + param(20+rptCount*2+srfCount+ii-1) / 2;
end;
y(1)		= param(20+rptCount*2+srfCount+ix+1) / 2 + pmlMinY;
for jj = 2: 1: jx
	y(jj)	= param(20+rptCount*2+srfCount+ix+jj) / 2 + y(jj-1) + param(20+rptCount*2+srfCount+ix+jj-1) / 2;
end;
z(1)		= param(20+rptCount*2+srfCount+ix+jx+1) / 2 + pmlMinZ;
for kk = 2: 1: kx
	z(kk)	= param(20+rptCount*2+srfCount+ix+jx+kk) / 2 + z(kk-1) + param(20+rptCount*2+srfCount+ix+jx+kk-1) / 2;
end;

if nargin >= 6;
	dfr	= dframe;
end;

tt	= 0;
strp	= [str '_p_' num2str(sn)];
stru	= [str '_u_' num2str(sn)];

vidObj	= VideoWriter([str '_' int2str(sn) '.mp4'], 'MPEG-4');
%vidObj	= VideoWriter([str '_' int2str(sn) '.avi'], 'Uncompressed AVI');
open(vidObj);

set(gca, 'NextPlot', 'replacechildren')

for t = dfr: dfr: tx;

	tt	= tt + 1;

	if 		t <= 9;			fid	= fopen([strp '_0000000' num2str(t)], 'r');
	elseif 	t <= 99;		fid	= fopen([strp '_000000' num2str(t)], 'r');
	elseif 	t <= 999;		fid	= fopen([strp '_00000' num2str(t)], 'r');
	elseif	t <= 9999;		fid	= fopen([strp '_0000' num2str(t)], 'r');
	elseif 	t <= 99999;		fid	= fopen([strp '_000' num2str(t)], 'r');
	elseif 	t <= 999999; 	fid	= fopen([strp '_00' num2str(t)], 'r');
	elseif	t <= 9999999;	fid	= fopen([strp '_0' num2str(t)], 'r');
	else; 					fid	= fopen([strp '_' num2str(t)], 'r');
	end;

	%linear
	% 0J[o[̒F̒ɁiꂪȂ0ɐFςj
	pave	= (pmin - pmax) / (cscale / 2);
	if		srfs(sn+1) == 0; p	= fread(fid, [kx jx], 'float64') + pave;
	elseif	srfs(sn+1) == 1; p	= fread(fid, [kx ix], 'float64') + pave;
	elseif	srfs(sn+1) == 2; p	= fread(fid, [jx ix], 'float64') + pave;
	end;

%	%deciBel
%	if		srfs(sn+1) == 0; p	= fread(fid, [kx jx], 'float64');
%	elseif	srfs(sn+1) == 1; p	= fread(fid, [kx ix], 'float64');
%	elseif	srfs(sn+1) == 2; p	= fread(fid, [jx ix], 'float64');
%	end;
%	for ii = 1: size(p,1);
%		for jj = 1: size(p,2);
%			if p(ii,jj)	== 0; p(ii,jj)	= pmin;
%			elseif p(ii,jj) == 1e30; p(ii,jj)	= Inf;
%			else; p(ii,jj)	= 20 * log10(abs(p(ii,jj)) / 2e-5);
%			end;
%		end;
%	end;

%	%ő̂ɂȂȂꍇ
%	sizep	= size(p);
%	for i = 1 : sizep(1);
%		for j = 1 : sizep(2);
%			if p(i,j) > pmax;
%			p(i,j) = Inf;
%			end;
%		end;
%	end;

	fclose(fid);

	if 		t <= 9;			fid = fopen([stru '_0000000' num2str(t)], 'r');
	elseif 	t <= 99;		fid = fopen([stru '_000000' num2str(t)], 'r');
	elseif 	t <= 999; 		fid = fopen([stru '_00000' num2str(t)], 'r');
	elseif	t <= 9999;		fid = fopen([stru '_0000' num2str(t)], 'r');
	elseif 	t <= 99999;		fid = fopen([stru '_000' num2str(t)], 'r');
	elseif 	t <= 999999; 	fid = fopen([stru '_00' num2str(t)], 'r');
	elseif	t <= 9999999;	fid = fopen([stru '_0' num2str(t)], 'r');
	else; 					fid = fopen([stru '_' num2str(t)], 'r');
	end

	v	= fread(fid,[4 inf],'float64');
	if(size(v,2) ~= 0);
		v(3,:)	= v(3,:) * ratio;
		v(4,:)	= v(4,:) * ratio;
		wv		= 10000 * ones(1, size(v,2));
		vw		= zeros(1, size(v,2));
		xv		= v(1,:);
		zv		= v(2,:);
		vx		= v(3,:);
		vz		= v(4,:);
	end;

	fclose(fid);

	if		srfs(sn+1) == 0; surf(y,z,p);
	elseif	srfs(sn+1) == 1; surf(x,z,p);
	elseif	srfs(sn+1) == 2; surf(x,y,p);
	end;

	shading interp; colormap(jet(cscale));
	axis equal;

	if nargin == 10;
		xmin	= xnx;
		ymin	= ynx;
		xmax	= xmx;
		ymax	= ymx;
	else
		if		srfs(sn+1) == 0;
			xmin	= minY;
			ymin	= minZ;
			xmax	= maxY;
			ymax	= maxZ;
		elseif	srfs(sn+1) == 1;
			xmin	= minX;
			ymin	= minZ;
			xmax	= maxX;
			ymax	= maxZ;
		elseif	srfs(sn+1) == 2;
			xmin	= minX;
			ymin	= minY;
			xmax	= maxX;
			ymax	= maxY;
		end;
	end;

	axis([xmin, xmax, ymin, ymax, -100000000000000, 100000000000000]);
	set(gca, 'clim', [pmin,pmax], 'view', [0 90]);
	colorbar;
	if		srfs(sn+1) == 0;
		title(['yz_' str '-' num2str(t)], 'Interpreter', 'none');
	elseif	srfs(sn+1) == 1;
		title(['xz_' str '-' num2str(t)], 'Interpreter', 'none');
	elseif	srfs(sn+1) == 2;
		title(['xy_' str '-' num2str(t)], 'Interpreter', 'none');
	end;
	hold on;
	if size(v,2)~=0;
		quiver3(xv, zv, wv, vx, vz, vw, 0, 'k', 'LineWidth', 1.5);
	end;
	grid off;
	hold off;

	thand	= text(xmax, ymin, 10001, [sprintf('%8.2f', dt*t*1000) ' ms']);
	set(thand, 'Fontname', 'Times New Roman', 'FontUnits', 'normalized', 'FontSize', 0.06, 'HorizontalAlignment', 'right', 'VerticalAlignment', 'bottom');
	
	currFrame	= getframe(gca);
	writeVideo(vidObj, currFrame);

end;

close(vidObj);

end