%Function M-file: vafdtd_graph.m
%
%Usage		: [ti, sg, fr, am, fs]	= vafdtd_graph(str1, istr1, ...)
%Version	: 1.1.04 (2024.10.6)
%
%ti			: Time Vector of First Argument Data
%sg			: Signal Vector of First Argument Data
%fr			: Frequency Vector of First Argument Data
%am			: Amplitude Vector of First Argument Data
%fs			: Sampling Frequency
%str		: Sequence File Name
%istr		: Position Name

function [ti, sg, fr, am, fa]	= vafdtd_graph(varargin);

col = [	122/255  16/255 228/255;
		 20/255  43/255 140/255;
		216/255  41/255   0/255;
		  0/255 127/255   0/255;
		222/255 125/255   0/255;
		 11/255 132/255 199/255;
		174/255 119/255   0/255 ];

ima	= nargin / 2;

for i = 1 : ima;
	st1{i}	= varargin{2 * i - 1};
	st2{i}	= varargin{2 * i};
	st{i}	= [st1{i} '_' st2{i}];
end;

figure;

for i = 1 : ima;

	fid1	= fopen([st1{i} '.ini'], 'r');
	fid2	= fopen([st{i}], 'r');

	param	= fread(fid1, [inf], 'float64');
	fclose(fid1);

	dt		= param(1);
	if(isempty(strfind(st2{i}, 'rpt')));
		timeskip	= 1;
	else
		timeskip	= param(19+param(19)+str2num(strrep(st2{i}, 'rpt', ''))+1);
	end;

	p	= fread(fid2, [inf], 'float64');

	fclose(fid2);

	dsize	= length(p);
	tmax	= dt * dsize;
	t		= (dt * timeskip) * [0:dsize - 1].' * 1000;
	fs(i)	= 1 / (dt * timeskip);
	df		= fs(i) / dsize;
	f		= df * [0:dsize - 1].';

%`
%	amp		= 20 * log10(abs(fft(p) * (dt * timeskip)) / 2e-5 / tmax);

%njO
%	han		= hanning(length(p) * 2);
%	han		= han(length(p) + 1:end);
%	amp		= 20 * log10(abs(fft(p .* han) * (dt * timeskip)) / 2e-5 / tmax);

%n~O	
	ham		= hamming(length(p) * 2);
	ham		= ham(length(p) + 1:end);
	amp		= 20 * log10(abs(fft(p .* ham) * (dt * timeskip)) / 2e-5 / tmax);

	if(max(amp) ~= -Inf);
		ampm(i)	= max(amp);
	else;
		ampm(i)	= 0;
	end;

	subplot(2, 1, 1);
	plot(t, p, 'Color', col(mod(i,7)+1, :), 'LineWidth', 0.5); hold on;
	subplot(2, 1, 2);
	semilogx(f, amp, 'o-', 'MarkerSize',2, 'Color', col(mod(i,7)+1, :), 'LineWidth', 0.5); hold on;

	ti	= t / 1000;
	sg	= p;
	fr	= f;
	am	= amp;
	fa	= fs(i);

end;

fsmin	= min(fs);
ampmax	= max(ampm);

subplot(2, 1, 1);
grid on;
set(gca, 'FontName', 'Times New Roman');
set(gca, 'FontSize', 16);
xlabel('Time [ms]');
ylabel('Amplitude');
title('Time Domain', 'Interpreter', 'none');
legend(st{:});
set(legend, 'Interpreter', 'none');

subplot(2, 1, 2);
grid on;
set(gca, 'FontName', 'Times New Roman');
set(gca, 'FontSize', 16);
xlabel('Frequency [Hz]');
ylabel('Amplitude [dB]');
xa	= 15.625 * 2 .^ [0:20];
set(gca, 'XTick', xa);
title('Frequency Domain', 'Interpreter', 'none');
legend(st{:});
set(legend, 'Interpreter', 'none');
set(gca, 'XLim', [15.625 8000]);
set(gca, 'XMinorGrid', 'off', 'XMinorTick', 'off');

hold off;

end